(function() {
  var Client,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  Client = (function() {

    Client.prototype.path = null;

    Client.prototype.apiServer = "http://" + window.location.host + "/protocol";

    Client.prototype.preview = null;

    Client.prototype.searchForm = null;

    Client.prototype.candidateSet = null;

    Client.prototype.canBeUpdated = true;

    function Client(editor, preview) {
      var searchListener,
        _this = this;
      this.editor = editor;
      this.closeSearchFrom = __bind(this.closeSearchFrom, this);

      this.focusSearchForm = __bind(this.focusSearchForm, this);

      this.openPreviousFocusedDiff = __bind(this.openPreviousFocusedDiff, this);

      this.openNextFocusedDiff = __bind(this.openNextFocusedDiff, this);

      this.openPreviousCandidate = __bind(this.openPreviousCandidate, this);

      this.openNextCandidate = __bind(this.openNextCandidate, this);

      this.onKeyPress = __bind(this.onKeyPress, this);

      this.onChangeFocusedResult = __bind(this.onChangeFocusedResult, this);

      this.setSearchResult = __bind(this.setSearchResult, this);

      this.apply = __bind(this.apply, this);

      this.update = __bind(this.update, this);

      this.commit = __bind(this.commit, this);

      this.restart = __bind(this.restart, this);

      this.quit = __bind(this.quit, this);

      this.save = __bind(this.save, this);

      this.open = __bind(this.open, this);

      this.apiUrl = __bind(this.apiUrl, this);

      searchListener = new SearchListener(function(index) {
        return _this.onChangeFocusedResult(index);
      }, function(text) {
        return _this.update(text);
      }, function(e) {
        return _this.onKeyPress(e);
      });
      this.preview = new Preview(preview);
      this.searchForm = new SearchForm(this.editor.container, searchListener);
      this.editor.on("change", function(e) {
        console.log(e);
        if (_this.canBeUpdated) {
          _this.searchForm.clear();
          return _this.update(null);
        }
      });
      this.editor.commands.addCommand({
        name: "focusSearchForm",
        bindKey: {
          win: "alt-S",
          linux: "alt-S"
        },
        exec: function(editor) {
          return _this.focusSearchForm();
        }
      });
      this.editor.commands.addCommand({
        name: "openNextCandidate",
        bindKey: {
          win: "Ctrl-space",
          linux: "Ctrl-space"
        },
        exec: function(editor) {
          return _this.openNextCandidate();
        }
      });
      this.editor.commands.addCommand({
        name: "Preview",
        bindKey: {
          win: "alt-P",
          linux: "alt-P"
        },
        exec: function(editor) {
          return _this.commit();
        }
      });
    }

    Client.prototype.apiUrl = function(api) {
      return "" + this.apiServer + "?api=" + api;
    };

    Client.prototype.open = function(path) {
      var _this = this;
      this.canBeUpdated = false;
      this.path = path;
      return $.post(this.apiUrl("Open"), JSON.stringify(path), function(data, status, jq) {
        _this.editor.getSession().setValue(data.text);
        _this.canBeUpdated = true;
        return _this.commit();
      });
    };

    Client.prototype.save = function() {
      return $.post(this.apiUrl("Save"), JSON.stringify(this.path));
    };

    Client.prototype.quit = function() {
      return $.post(this.apiUrl("Quit"), "");
    };

    Client.prototype.restart = function() {
      $.post(this.apiUrl("Restart"), "");
      return document.location.reload();
    };

    Client.prototype.commit = function() {
      var file,
        _this = this;
      file = {
        fileName: this.path,
        text: this.editor.getSession().getValue()
      };
      $.post(this.apiUrl("Commit"), JSON.stringify(file), function(data, status, jq) {
        return _this.update(null);
      });
      return this.preview.reload("http://" + window.location.host + "/preview?file=" + file.fileName);
    };

    Client.prototype.update = function(text) {
      var request,
        _this = this;
      request = {
        file: {
          "fileName": this.path,
          "text": this.editor.getSession().getValue()
        },
        textToSearch: text,
        regionToSearch: null
      };
      return $.post(this.apiUrl("UpdateCandidates"), JSON.stringify(request), function(data, status, jq) {
        if (_this.candidateSet != null) {
          _this.candidateSet.clear();
        }
        _this.candidateSet = new CandidateSet(_this.editor, data, _this.apply, _this.onKeyPress);
        if (text != null) {
          return _this.setSearchResult(data.focusableList);
        } else {
          return _this.setSearchResult(null);
        }
      });
    };

    Client.prototype.apply = function(id) {
      var request,
        _this = this;
      this.canBeUpdated = false;
      request = {
        file: {
          fileName: this.path,
          text: this.editor.getSession().getValue()
        },
        id: id
      };
      return $.post(this.apiUrl("ApplyCandidate"), JSON.stringify(request), function(data, status, jq) {
        var aceDiff, delta, document, _i, _len, _ref;
        aceDiff = new AceDiff(_this.editor, data.diff);
        _ref = aceDiff.deltas;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          delta = _ref[_i];
          document = _this.editor.getSession().getDocument();
          document.applyDelta(delta);
          console.log(_this.editor.getSession().getDocument().getLine(41));
        }
        _this.canBeUpdated = true;
        _this.update(null);
        return _this.preview.reload("http://" + window.location.host + "/preview?file=" + request.file.fileName);
      });
    };

    Client.prototype.setSearchResult = function(list) {
      if (list != null) {
        this.searchForm.setNumberOfSearchResult(list.length);
        return this.searchForm.focus(0);
      } else {
        return this.searchForm.setNumberOfSearchResult(null);
      }
    };

    Client.prototype.onChangeFocusedResult = function(index) {
      if (this.candidateSet != null) {
        return this.candidateSet.focusToList(index);
      }
    };

    Client.prototype.onKeyPress = function(e) {
      if (e.ctrlKey) {
        switch (e.keyCode) {
          case Constants.KeyCode.up:
            return this.openPreviousFocusedDiff();
          case Constants.KeyCode.down:
            return this.openNextFocusedDiff();
          case Constants.KeyCode.space:
            return this.openNextCandidate();
          default:
            switch (e.charCode) {
              case Constants.KeyCode.space:
                return this.openNextCandidate();
            }
        }
      } else if (e.altKey) {
        switch (e.keyCode) {
          case Constants.KeyCode.s:
            return this.openSearchForm();
        }
      } else {
        switch (e.keyCode) {
          case Constants.KeyCode.escape:
            this.editor.focus();
            return this.closeSearchFrom();
        }
      }
    };

    Client.prototype.openNextCandidate = function() {
      if (this.candidateSet != null) {
        return this.candidateSet.focusToNextCandidate();
      }
    };

    Client.prototype.openPreviousCandidate = function() {};

    Client.prototype.openNextFocusedDiff = function() {
      if (this.candidateSet != null) {
        return this.candidateSet.focusToNextBallon();
      }
    };

    Client.prototype.openPreviousFocusedDiff = function() {
      if (this.candidateSet != null) {
        return this.candidateSet.focusToPreviousBallon();
      }
    };

    Client.prototype.focusSearchForm = function() {
      return this.searchForm.open();
    };

    Client.prototype.closeSearchFrom = function() {
      return this.searchForm.close();
    };

    return Client;

  })();

  window.Client = Client;

}).call(this);
